@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '2.007';
+our $VERSION = '2.008';
 
 use List::MoreUtils qw( first_index );
 
@@ -31,25 +31,13 @@ sub col_function {
     my ( $sf, $sql, $stmt_type ) = @_;
     my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @functions = ( qw( Epoch_to_Date Bit_Length Truncate Char_Length Epoch_to_DateTime ) );
-    my $cols_type = '';
-    if ( $sql->{select_type} eq '*' ) {
+    if ( @{$sql->{group_by_cols}} + @{$sql->{aggr_cols}} + @{$sql->{chosen_cols}} == 0 ) {
         @{$sql->{chosen_cols}} = @{$sql->{cols}};
-        $cols_type = 'chosen_cols';
     }
-    elsif ( $sql->{select_type} eq 'chosen_cols' ) {
-        $cols_type = 'chosen_cols';
-    }
-    if ( $cols_type eq 'chosen_cols' ) {
-        if ( ! @{$sql->{orig_chosen_cols}} ) {
-            @{$sql->{orig_chosen_cols}} = @{$sql->{'chosen_cols'}};
-        }
-    }
-    else {
-        if ( @{$sql->{aggr_cols}} && ! @{$sql->{orig_aggr_cols}} ) {
-            @{$sql->{orig_aggr_cols}} = @{$sql->{'aggr_cols'}};
-        }
-        if ( @{$sql->{group_by_cols}} && ! @{$sql->{orig_group_by_cols}} ) {
-            @{$sql->{orig_group_by_cols}} = @{$sql->{'group_by_cols'}};
+    for my $col_type ( qw( chosen_cols aggr_cols group_by_cols ) ) {
+        my $col_type_orig = 'orig_' . $col_type;
+        if ( @{$sql->{$col_type}} && ! @{$sql->{$col_type_orig}} ) {
+            @{$sql->{$col_type_orig}} = @{$sql->{$col_type}};
         }
     }
     my $changed = 0;
@@ -57,13 +45,7 @@ sub col_function {
     COL_SCALAR_FUNC: while ( 1 ) {
         my $default = 0;
         my @pre = ( undef, $sf->{i}{_confirm} );
-        my @cols;
-        if ( $cols_type eq 'chosen_cols' ) {
-            @cols = @{$sql->{chosen_cols}};
-        }
-        else {
-            @cols = ( @{$sql->{aggr_cols}}, @{$sql->{group_by_cols}} );
-        }
+        my @cols = ( @{$sql->{group_by_cols}}, @{$sql->{aggr_cols}}, @{$sql->{chosen_cols}} );
         my $choices = [ @pre, map( "- $_", @cols ) ];
         $ax->print_sql( $sql, [ $stmt_type ] );
         # Choose
@@ -78,19 +60,21 @@ sub col_function {
             if ( ! $changed ) {
                 return;
             }
-            $sql->{select_type} = 'chosen_cols' if $sql->{select_type} eq '*'; # makes the changes visible
             return 1; # return $tmp
         }
         ( my $qt_col = $choices->[$idx] ) =~ s/^\-\s//;
         $idx -= @pre;
-        if ( $cols_type ne 'chosen_cols' ) {
-            if ( $idx - @{$sql->{aggr_cols}} >= 0 ) { # chosen a "group by" col
-                $idx -= @{$sql->{aggr_cols}};
-                $cols_type = 'group_by_cols';
-            }
-            else {
-                $cols_type = 'aggr_cols';
-            }
+        my $cols_type;
+        if ( $idx <= $#{$sql->{group_by_cols}} ) {
+            $cols_type = 'group_by_cols';
+        }
+        elsif ( $idx <= $#{$sql->{aggr_cols}} ) {
+            $idx -= @{$sql->{group_by_cols}};
+            $cols_type = 'aggr_cols';
+        }
+        else {
+            $idx -= @{$sql->{group_by_cols}} + @{$sql->{aggr_cols}};
+            $cols_type = 'chosen_cols';
         }
         # reset col to original, if __col_function is called on a already modified col:
         if ( $sql->{$cols_type}[$idx] ne $sql->{'orig_' . $cols_type}[$idx] ) {
